"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var inversify_1 = require("inversify");
var Registrar = (function () {
    function Registrar(container, server, dispatcher) {
        this.container = container;
        this.server = server;
        this.dispatcher = dispatcher;
    }
    Registrar.prototype.registerRoutes = function (routes) {
        var _this = this;
        _.each(routes, function (route) { return _this.registerRoute(route); });
    };
    Registrar.prototype.registerRoute = function (route) {
        if (_.has(route, "actionObject")) {
            this.registerActionRoute(route);
        }
        else if (_.has(route, "directory")) {
            this.registerDirectoryRoute(route);
        }
        else if (_.has(route, "file")) {
            this.registerFileRoute(route);
        }
        else {
            this.registerHapiRoute(route);
        }
    };
    Registrar.prototype.registerActionRoute = function (route) {
        var symbol = this.bindAction(route);
        var dispatcher = this.dispatcher;
        var actionHandler = function (request, reply) {
            dispatcher.dispatch(request, reply, route, symbol);
        };
        this.registerHapiRoute({
            path: route.path,
            method: route.method || "GET",
            handler: actionHandler,
        });
    };
    Registrar.prototype.bindAction = function (route) {
        var actionSymbol = Symbol(route.method + "#" + route.path);
        inversify_1.decorate(inversify_1.injectable(), route.actionObject);
        var bind = this.container
            .bind(actionSymbol)
            .to(route.actionObject)
            .inTransientScope();
        if (!_.isEmpty(route.injections)) {
            this.bindConstructorParameters(route.actionObject, route.injections);
        }
        return actionSymbol;
    };
    Registrar.prototype.bindConstructorParameters = function (staticType, injections) {
        injections.forEach(function (injection, position) {
            if (_.isArray(injection)) {
                inversify_1.decorate(inversify_1.multiInject(injection[0]), staticType, position);
            }
            else {
                inversify_1.decorate(inversify_1.inject(injection), staticType, position);
            }
        });
    };
    Registrar.prototype.registerDirectoryRoute = function (route) {
        this.registerHapiRoute({
            method: route.method || "GET",
            path: route.path || "/{path*}",
            handler: {
                directory: {
                    path: route.directory,
                    index: true,
                },
            },
        });
    };
    Registrar.prototype.registerFileRoute = function (route) {
        this.registerHapiRoute({
            method: route.method || "GET",
            path: route.path || "/{path*}",
            handler: {
                file: route.file,
            },
        });
    };
    Registrar.prototype.registerHapiRoute = function (route) {
        this.server.route(route);
    };
    return Registrar;
}());
exports.Registrar = Registrar;
//# sourceMappingURL=Registrar.js.map